@@ -36,7 +36,7 @@ def supported_formats():
     return formats
 
 def open(filename):
-    """Open the specified file and return a File instance, or None."""
+    """Open the specified file and return a File instance with the appropriate format handler, or None."""
     i = filename.rfind(".")
     if i < 0:
         return None
@@ -150,22 +150,26 @@ from picard.formats.apev2 import (
     MusepackFile,
     OptimFROGFile,
     WavPackFile,
+    TAKFile,
     )
 register_format(MusepackFile)
 register_format(WavPackFile)
 register_format(OptimFROGFile)
 register_format(MonkeysAudioFile)
+register_format(TAKFile)
 
 from picard.formats.vorbis import (
     FLACFile,
     OggFLACFile,
     OggSpeexFile,
     OggVorbisFile,
+    OggAudioFile,
     )
 register_format(FLACFile)
 register_format(OggFLACFile)
 register_format(OggSpeexFile)
 register_format(OggVorbisFile)
+register_format(OggAudioFile)
 
 try:
     from picard.formats.mp4 import MP4File
