@@ -25,7 +25,6 @@ except ImportError:
 from picard import version_string
 from picard.const import MUSICDNS_KEY
 from picard.util import encode_filename, partial, call_next
-from picard.util.thread import proxy_to_main
 
 
 class OFA(QtCore.QObject):
@@ -55,7 +54,7 @@ class OFA(QtCore.QObject):
         for decoder in self._decoders:
             decoder.done()
 
-    def create_fingerprint(self, filename):
+    def calculate_fingerprint(self, filename):
         """Decode the specified file and calculate a fingerprint."""
         if ofa is None:
             return None, 0
@@ -90,8 +89,8 @@ class OFA(QtCore.QObject):
         except (KeyError):
             # The file has been removed. do nothing
             return
-        
-        if result is None or error is not None:
+
+        if result is None or result[0] is None or error is not None:
             next(file, result=None)
             return
         fingerprint, length = result
@@ -114,25 +113,7 @@ class OFA(QtCore.QObject):
             gnr=file.metadata["genre"],
             yrr=file.metadata["date"][:4])
 
-    def _calculate_fingerprint(self, filename):
-        self.tagger.window.set_statusbar_message(N_("Creating fingerprint for file %s..."), filename)
-        filename = encode_filename(filename)
-        for decoder in self._decoders:
-            self.log.debug("Decoding using %r...", decoder.__name__)
-            try:
-                result = decoder.decode(filename)
-            except Exception:
-                continue
-            if result:
-                self.log.debug("Fingerprinting...")
-                data, samples, sample_rate, stereo, duration = result
-                fingerprint = ofa.create_print(data, samples, sample_rate, stereo)
-                if fingerprint:
-                    return fingerprint, duration
-                else:
-                    break
-
-    def analyze(self, file, next, thread_pool):
+    def analyze(self, file, next):
         # return cached PUID
         puids = file.metadata.getall('musicip_puid')
         if puids:
@@ -150,5 +131,3 @@ class OFA(QtCore.QObject):
             return
         # no PUID
         next(result=None)
-
-
