@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from PyQt4 import QtCore, QtGui
+from PyQt4 import QtCore, QtGui, QtNetwork
 
 import os.path
 import sys
@@ -40,11 +40,12 @@ from picard.ui.passworddialog import PasswordDialog
 from picard.util import icontheme, webbrowser2, find_existing_path
 from picard.util.cdrom import get_cdrom_drives
 from picard.plugin import ExtensionPoint
+import picard.musicdns
 
 ui_init = ExtensionPoint()
 def register_ui_init (function):
     ui_init.register(function.__module__, function)
-    
+
 class MainWindow(QtGui.QMainWindow):
 
     options = [
@@ -148,7 +149,7 @@ class MainWindow(QtGui.QMainWindow):
         size = self.config.persist["window_size"]
         self._desktopgeo = self.tagger.desktop().screenGeometry()
         if pos.x() > 0 and pos.y() > 0 and pos.x()+size.width() < self._desktopgeo.width() and pos.y()+size.height() < self._desktopgeo.height():
-            self.move(pos)        
+            self.move(pos)
         if size.width() <= 0 or size.height() <= 0:
             size = QtCore.QSize(780, 560)
         self.resize(size)
@@ -166,9 +167,8 @@ class MainWindow(QtGui.QMainWindow):
 
     def update_statusbar(self, num_pending_files):
         """Updates the status bar information."""
-        self.file_counts_label.setText(" Files: %d, Pending Files: %d " % (
-            self.tagger.num_files(),
-            num_pending_files))
+        self.file_counts_label.setText(_(" Files: %(files)d, Pending Files: %(pending)d ")
+            % {"files": self.tagger.num_files(), "pending": num_pending_files})
 
     def set_statusbar_message(self, message, *args, **kwargs):
         """Set the status bar message."""
@@ -251,8 +251,7 @@ class MainWindow(QtGui.QMainWindow):
         self.exit_action = QtGui.QAction(_(u"E&xit"), self)
         # TR: Keyboard shortcut for "Exit"
         self.exit_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+Q")))
-        self.connect(self.exit_action, QtCore.SIGNAL("triggered()"),
-                     self.close)
+        self.connect(self.exit_action, QtCore.SIGNAL("triggered()"), self.close)
 
         self.remove_action = QtGui.QAction(icontheme.lookup('list-remove'), _(u"&Remove"), self)
         self.remove_action.setStatusTip(_(u"Remove selected files/albums"))
@@ -305,6 +304,8 @@ class MainWindow(QtGui.QMainWindow):
 
         self.edit_tags_action = QtGui.QAction(icontheme.lookup('picard-edit-tags'), _(u"&Details..."), self)
         self.edit_tags_action.setEnabled(False)
+        # TR: Keyboard shortcut for "Details"
+        self.edit_tags_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+I")))
         self.connect(self.edit_tags_action, QtCore.SIGNAL("triggered()"), self.edit_tags)
 
         self.refresh_action = QtGui.QAction(icontheme.lookup('view-refresh', icontheme.ICON_SIZE_MENU), _("&Refresh"), self)
@@ -335,6 +336,7 @@ class MainWindow(QtGui.QMainWindow):
         self.connect(self.view_log_action, QtCore.SIGNAL("triggered()"), self.show_log)
 
         self.connect(self.tagger.xmlws, QtCore.SIGNAL("authentication_required"), self.show_password_dialog)
+        self.connect(self.tagger.xmlws, QtCore.SIGNAL("proxyAuthentication_required"), self.show_proxy_dialog)
 
     def toggle_rename_files(self, checked):
         self.config.setting["rename_files"] = checked
@@ -404,10 +406,8 @@ class MainWindow(QtGui.QMainWindow):
             self.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
         else:
             self.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
-        
         self.cd_lookup_action.setEnabled(len(get_cdrom_drives()) > 0)
 
-
     def create_toolbar(self):
         self.toolbar = toolbar = self.addToolBar(_(u"&Toolbar"))
         self.update_toolbar_style()
@@ -430,9 +430,9 @@ class MainWindow(QtGui.QMainWindow):
             button.setPopupMode(QtGui.QToolButton.MenuButtonPopup)
             button.setMenu(self.cd_lookup_menu)
 
+        toolbar.addAction(self.cluster_action)
         toolbar.addAction(self.autotag_action)
         toolbar.addAction(self.analyze_action)
-        toolbar.addAction(self.cluster_action)
         toolbar.addAction(self.edit_tags_action)
         toolbar.addAction(self.remove_action)
         self.search_toolbar = toolbar = self.addToolBar(_(u"&Search Bar"))
@@ -515,7 +515,7 @@ class MainWindow(QtGui.QMainWindow):
         elif len(dir_list) > 1:
             (parent, dir) = os.path.split(str(dir_list[0]))
             self.config.persist["current_directory"] = parent
-            
+
         for directory in dir_list:
             directory = unicode(directory)
             self.tagger.add_directory(directory)
@@ -558,7 +558,20 @@ class MainWindow(QtGui.QMainWindow):
         if sys.platform == "linux2":
             args.append("os=Linux")
         elif sys.platform == "win32":
-            args.append("os=Windows+XP")
+            import platform
+            release = platform.release()
+            if release in ("post2008Server", "7"):
+                args.append("os=Windows+7")
+            elif release in ("Vista", "2008Server"):
+                args.append("os=Windows+Vista")
+            elif release in ("XP", "2003Server"):
+                args.append("os=Windows+XP")
+            elif release in ("NT", "2000"):
+                args.append("os=Windows+NT%2F2000")
+            elif release in ("95", "98", "Me"):
+                args.append("os=Windows+95%2F98%2FMe")
+            else:
+                args.append("os=Windows+XP")
         elif sys.platform == "darwin":
             args.append("os=Mac+OS+X")
         webbrowser2.open("http://bugs.musicbrainz.org/newticket?" + "&".join(args))
@@ -601,7 +614,7 @@ class MainWindow(QtGui.QMainWindow):
         can_refresh = False
         can_autotag = False
         for obj in self.selected_objects:
-            if obj.can_analyze():
+            if picard.musicdns.ofa and obj.can_analyze():
                 can_analyze = True
             if obj.can_save():
                 can_save = True
@@ -611,10 +624,11 @@ class MainWindow(QtGui.QMainWindow):
                 can_edit_tags = True
             if obj.can_refresh():
                 can_refresh = True
-            if can_save and can_remove and can_edit_tags and can_refresh:
+            if obj.can_autotag():
+                can_autotag = True
+            if can_save and can_remove and can_edit_tags and can_refresh \
+                    and can_autotag:
                 break
-        # FIXME
-        can_autotag = can_remove
         self.remove_action.setEnabled(can_remove)
         self.save_action.setEnabled(can_save)
         self.edit_tags_action.setEnabled(can_edit_tags)
@@ -646,14 +660,14 @@ class MainWindow(QtGui.QMainWindow):
                     statusBar += _(" (Error: %s)") % obj.error
                 file = obj
             elif isinstance(obj, Track):
-                if len(obj.linked_files) == 1:
+                if obj.num_linked_files == 1:
                     file = obj.linked_files[0]
                     orig_metadata = file.orig_metadata
                     metadata = file.metadata
                     statusBar = "%s (%d%%)" % (file.filename, file.similarity * 100)
                     if file.state == file.ERROR:
                         statusBar += _(" (Error: %s)") % file.error
-                elif len(obj.linked_files) == 0:
+                elif obj.num_linked_files == 0:
                     metadata = obj.metadata
                 else:
                     metadata = obj.metadata
@@ -688,8 +702,12 @@ class MainWindow(QtGui.QMainWindow):
         else:
             self.file_browser.hide()
 
-    def show_password_dialog(self, host, port, authenticator):
-        dialog = PasswordDialog(authenticator, host, port, parent=self)
+    def show_password_dialog(self, reply, authenticator):
+        dialog = PasswordDialog(authenticator, reply, parent=self)
+        dialog.exec_()
+
+    def show_proxy_dialog(self, proxy, authenticator):
+        dialog = ProxyDialog(authenticator, proxy, parent=self)
         dialog.exec_()
 
     def autotag(self):
