@@ -30,9 +30,12 @@ from picard.ui.options import (
     general,
     interface,
     folksonomy,
+    ratings,
     matching,
     metadata,
-    naming,
+    releases,
+    moving,
+    renaming,
     plugins,
     proxy,
     scripting,
@@ -117,8 +120,7 @@ class OptionsDialog(QtGui.QDialog):
                 page.check()
             except OptionsCheckError, e:
                 self.ui.pages_tree.setCurrentItem(self.page_to_item[page.NAME])
-                dialog = QtGui.QMessageBox(QtGui.QMessageBox.Warning, e.title, e.message, QtGui.QMessageBox.Ok, self)
-                dialog.exec_()
+                page.display_error(e)
                 return
         for page in self.pages:
             page.save()
