@@ -21,8 +21,6 @@ from PyQt4 import QtCore, QtGui
 from picard.config import BoolOption, TextOption
 from picard.ui.options import OptionsPage, OptionsCheckError, register_options_page
 from picard.ui.ui_options_metadata import Ui_MetadataOptionsPage
-from picard.const import RELEASE_COUNTRIES
-import operator
 
 
 class MetadataOptionsPage(OptionsPage):
@@ -40,7 +38,10 @@ class MetadataOptionsPage(OptionsPage):
         BoolOption("setting", "release_ars", True),
         BoolOption("setting", "track_ars", False),
         BoolOption("setting", "folksonomy_tags", False),
-        TextOption("setting", "preferred_release_country", u""),
+        BoolOption("setting", "convert_punctuation", False),
+        BoolOption("setting", "standardize_tracks", False),
+        BoolOption("setting", "standardize_releases", False),
+        BoolOption("setting", "standardize_artists", False),
     ]
 
     def __init__(self, parent=None):
@@ -49,28 +50,30 @@ class MetadataOptionsPage(OptionsPage):
         self.ui.setupUi(self)
         self.connect(self.ui.va_name_default, QtCore.SIGNAL("clicked()"), self.set_va_name_default)
         self.connect(self.ui.nat_name_default, QtCore.SIGNAL("clicked()"), self.set_nat_name_default)
-        self.ui.preferred_release_country.addItem(N_("None"), QtCore.QVariant(""))
-        for country, name in sorted(RELEASE_COUNTRIES.items(), key=operator.itemgetter(1)):
-            self.ui.preferred_release_country.addItem(name, QtCore.QVariant(country))
 
     def load(self):
         self.ui.translate_artist_names.setChecked(self.config.setting["translate_artist_names"])
+        self.ui.convert_punctuation.setChecked(self.config.setting["convert_punctuation"])
         self.ui.release_ars.setChecked(self.config.setting["release_ars"])
         self.ui.track_ars.setChecked(self.config.setting["track_ars"])
         self.ui.folksonomy_tags.setChecked(self.config.setting["folksonomy_tags"])
-        current_release_country = QtCore.QVariant(self.config.setting["preferred_release_country"])
-        self.ui.preferred_release_country.setCurrentIndex(self.ui.preferred_release_country.findData(current_release_country))
         self.ui.va_name.setText(self.config.setting["va_name"])
         self.ui.nat_name.setText(self.config.setting["nat_name"])
+        self.ui.standardize_tracks.setChecked(self.config.setting["standardize_tracks"])
+        self.ui.standardize_releases.setChecked(self.config.setting["standardize_releases"])
+        self.ui.standardize_artists.setChecked(self.config.setting["standardize_artists"])
 
     def save(self):
         self.config.setting["translate_artist_names"] = self.ui.translate_artist_names.isChecked()
+        self.config.setting["convert_punctuation"] = self.ui.convert_punctuation.isChecked()
         self.config.setting["release_ars"] = self.ui.release_ars.isChecked()
         self.config.setting["track_ars"] = self.ui.track_ars.isChecked()
         self.config.setting["folksonomy_tags"] = self.ui.folksonomy_tags.isChecked()
-        self.config.setting["preferred_release_country"] = self.ui.preferred_release_country.itemData(self.ui.preferred_release_country.currentIndex()).toString()
         self.config.setting["va_name"] = self.ui.va_name.text()
         self.config.setting["nat_name"] = self.ui.nat_name.text()
+        self.config.setting["standardize_tracks"] = self.ui.standardize_tracks.isChecked()
+        self.config.setting["standardize_releases"] = self.ui.standardize_releases.isChecked()
+        self.config.setting["standardize_artists"] = self.ui.standardize_artists.isChecked()
 
     def set_va_name_default(self):
         self.ui.va_name.setText(self.options[0].default)
