@@ -27,6 +27,33 @@ from encodings import rot_13;
 from string import Template
 
 
+def asciipunct(s):
+    mapping = {
+        u"…": u"...",
+        u"‘": u"'",
+        u"’": u"'",
+        u"‚": u"'",
+        u"“": u"\"",
+        u"”": u"\"",
+        u"„": u"\"",
+        u"′": u"'",
+        u"″": u"\"",
+        u"‹": u"<",
+        u"›": u">",
+        u"«": u"<<",
+        u"»": u">>",
+        u"‐": u"-",
+        u"‒": u"-",
+        u"–": u"-",
+        u"−": u"-",
+        u"—": u"-",
+        u"―": u"--",
+    }
+    for orig, repl in mapping.iteritems():
+        s = s.replace(orig, repl)
+    return s
+
+
 def needs_read_lock(func):
     """Adds a read lock around ``func``.
 
@@ -123,7 +150,7 @@ def set_io_encoding(encoding):
 def encode_filename(filename):
     """Encode unicode strings to filesystem encoding."""
     if isinstance(filename, unicode):
-        if os.path.supports_unicode_filenames:
+        if os.path.supports_unicode_filenames and sys.platform != "darwin":
             return filename
         else:
             return filename.encode(_io_encoding, 'replace')
@@ -145,7 +172,7 @@ def format_time(ms):
     if ms == 0:
         return "?:??"
     else:
-        return "%d:%02d" % (ms / 60000, (ms / 1000) % 60)
+        return "%d:%02d" % (round(ms / 1000.0) / 60, round(ms / 1000.0) % 60)
 
 def sanitize_date(datestr):
     """Sanitize date format.
@@ -197,7 +224,7 @@ def replace_win32_incompat(string, repl=u"_"):
 _re_non_alphanum = re.compile(r'\W+', re.UNICODE)
 def strip_non_alnum(string):
     """Remove all non-alphanumeric characters from ``string``."""
-    return _re_non_alphanum.sub(u" ", string)
+    return _re_non_alphanum.sub(u" ", string).strip()
 
 _re_slashes = re.compile(r'[\\/]', re.UNICODE)
 def sanitize_filename(string, repl="_"):
@@ -207,7 +234,7 @@ def make_short_filename(prefix, filename, max_path_length=240, max_length=200,
                         mid_length=32, min_length=2):
     """
     Attempts to shorten the file name to the maximum allowed length.
-    
+
     max_path_length: The maximum length of the complete path.
     max_length: The maximum length of a single file or directory name.
     mid_length: The medium preferred length of a single file or directory.
@@ -317,11 +344,23 @@ def call_next(func):
     return func_wrapper
 
 
-_puid_format = Template('$h{8}-$h$l-$h$l-$h$l-$h{12}').safe_substitute(h='[0-9a-fA-F]', l='{4}')
-_re_puid_val = re.compile(_puid_format)
-def puid_validate(string):
-    return _re_puid_val.match(string)
+_mbid_format = Template('$h{8}-$h$l-$h$l-$h$l-$h{12}').safe_substitute(h='[0-9a-fA-F]', l='{4}')
+_re_mbid_val = re.compile(_mbid_format)
+def mbid_validate(string):
+    return _re_mbid_val.match(string)
 
 
 def rot13(input):
     return u''.join(unichr(rot_13.encoding_map.get(ord(c), ord(c))) for c in input)
+
+
+def load_release_type_scores(setting):
+    scores = {}
+    values = setting.split()
+    for i in range(0, len(values), 2):
+        scores[values[i]] = float(values[i+1]) if i+1 < len(values) else 0.0
+    return scores
+
+
+def save_release_type_scores(scores):
+    return " ".join(["%s %.2f" % v for v in scores.iteritems()])
